/*
 * C6Run_mem.h
 */

/*
 * Copyright (C) 2010 Texas Instruments Incorporated - http://www.ti.com/ 
 */

/* 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/************************************************************************
*  Header file for dispatch.c module.                                   *
************************************************************************/

#ifndef _C6RUN_MEM_H_
#define _C6RUN_MEM_H_

#include <stddef.h>
#include <stdint.h>
#include <stdbool.h>
#include <string.h>

// Prevent C++ name mangling
#ifdef __cplusplus
extern "C" {
#endif


/***********************************************************
* Global Macro Declarations                                *
***********************************************************/ 


/***********************************************************
* Global Typedef declarations                              *
***********************************************************/

// C6RUN_Buffer
typedef struct _C6RUN_MEM_BUFFER_
{
  struct _C6RUN_MEM_BUFFER_  *next;
  uint32_t                   magic;
}
C6RUN_MEM_Buffer, *pC6RUN_MEM_Buffer;


/***********************************************************
* Global Variable Declarations                             *
***********************************************************/

extern volatile bool  C6RUN_MEM_initialized;
extern void          *C6RUN_MEM_virtBaseAddr;
extern void          *C6RUN_MEM_physBaseAddr;


/***********************************************************
* Global Function Declarations                             *
***********************************************************/

// Memory subsystem initialization routines
extern int      C6RUN_MEM_init (void);
extern void     C6RUN_MEM_exit (void);

// Memory buffer management routines
extern void    *C6RUN_MEM_malloc(size_t size);
extern void    *C6RUN_MEM_calloc(size_t nelem, size_t elsize);
extern void    *C6RUN_MEM_realloc(void *oldPtr,size_t size);
extern void    *C6RUN_MEM_memalign(size_t alignment, size_t size);
extern void     C6RUN_MEM_free(void* ptr);

// Memory cache management routines
extern void     C6RUN_MEM_globalInv( void );
extern void     C6RUN_MEM_globalWb( void );
extern void     C6RUN_MEM_inv(void *ptr,size_t size);
extern void     C6RUN_MEM_wb(void *ptr,size_t size);

// Memory helper routines
extern void    *C6RUN_MEM_lookupBuffer(void *ptr, size_t *buffSize);
extern void    *C6RUN_MEM_virt2Phys(void *virtPtr);
extern void    *C6RUN_MEM_phys2Virt(void *physPtr);


/***********************************************************
* End file                                                 *
***********************************************************/

#ifdef __cplusplus
}
#endif

#endif //_C6RUN_MEM_H_
